/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.data;

import java.util.Observable;

/**
 * Keeps track where the mouse is.
 * 
 * @author Thomas Abeel
 * 
 */
public class MouseModel extends Observable{

	/**
	 * The coordinate (nt) where to mouse is currently hovering. -1 when the
	 * mouse is outside of the track window.
	 */
	private int currentCoord;

	/**
	 * Set the current coordinate: the place (nt) where the mouse is currently
	 * hovering. -1 when the mouse is outside of the track window.
	 */
	public void setCurrentCoord(int currentCoord) {
		this.currentCoord = currentCoord;
		setChanged();
		notifyObservers();
	}

	/**
	 * Get the current coordinate: the place (nt) where the mouse is currently
	 * hovering. -1 when the mouse is outside of the track window.
	 * 
	 * @return currentCoord
	 */
	public int getCurrentCoord() {
		return currentCoord;
	}
}
