/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.explorer;

import java.awt.EventQueue;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JOptionPane;

import net.sf.genomeview.core.Configuration;
import net.sf.genomeview.data.Model;
import net.sf.genomeview.gui.components.ConnectionMonitor;

/**
 * 
 * @author Thomas Abeel
 * 
 */
public class GenomeExplorerManager implements Observer {
	GenomeExplorer bg;
	private Model model;

	private boolean autoMode = Configuration.getBoolean("general:enableGenomeExplorer");

	public GenomeExplorerManager(Model model) {
		bg = new GenomeExplorer(model);
		this.model = model;
		model.getGUIManager().registerGenomeExplorer(this);
		model.addObserver(this);
		model.getWorkerManager().addObserver(this);
	}

	public void setVisible(final boolean vis) {
		if(vis)
			autoMode = Configuration.getBoolean("general:enableGenomeExplorer");
		else
			autoMode = false;
		visi(vis);

	}

	private boolean firstUse = true;

	private void visi(final boolean vis) {
		EventQueue.invokeLater(new Runnable() {

			@Override
			public void run() {
				if(vis&&ConnectionMonitor.instance.offline()){
					autoMode=false;
					JOptionPane.showMessageDialog(model.getGUIManager().getParent(), "GenomeView has no internet access!\nYou need network connectivity to load data online.","Offline",JOptionPane.WARNING_MESSAGE);
				}else{
					bg.setVisible(vis);
				}
			}
		});

		if (vis && firstUse) {
			firstUse = false;
			EventQueue.invokeLater(new Runnable() {

				@Override
				public void run() {
					bg.scollToTop();
				}
			});

		}
	}

	@Override
	public void update(Observable o, Object arg) {
		if (!autoMode)
			return;

		if (model.getWorkerManager().runningJobs()>0||model.entries().size() > 0) {
			if (bg.isVisible())
				visi(false);
		} else {
			if (!bg.isVisible()) {
				visi(true);

			}
		}

	}
}
