/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.information;

import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.logging.Logger;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;

import net.sf.genomeview.data.Model;
import net.sf.genomeview.data.TrackList;
import net.sf.genomeview.gui.viztracks.Track;

/**
 * 
 * @author Thomas Abeel
 * 
 */
public class TrackTable extends JTable {
	private Logger log = Logger.getLogger(TrackTable.class.getCanonicalName());
	private Model model;
	private static final long serialVersionUID = 2680194422003453639L;

	public TrackTable(final Model model) {
		super(new TrackTableModel(model));
		this.model=model;
		setSelectionModel(new TrackSelectionModel());
		
		setTableHeader(null);
		setUI(new DragDropRowTableUI(model));
		/* Set column widths */
	
		getColumnModel().getColumn(0).setPreferredWidth(20);
		getColumnModel().getColumn(0).setMaxWidth(50);
		getColumnModel().getColumn(2).setPreferredWidth(20);
		getColumnModel().getColumn(2).setMaxWidth(50);

		ToolTipManager.sharedInstance().setInitialDelay(0);
		

	}

	public Component prepareRenderer(TableCellRenderer renderer, int rowIndex, int vColIndex) {
		Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
		if (c instanceof JComponent) {
			JComponent jc = (JComponent) c;
			String longDisplay= model.getTrackList().get(rowIndex).config().displayName();
			jc.setToolTipText(longDisplay);
		}
		return c;
	}

	

}
