/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.search;

import java.util.ArrayList;

import net.sf.genomeview.data.Model;
import net.sf.jannot.Entry;

/**
 * 
 * @author Thomas Abeel
 * 
 */
class EntrySearchResultModel extends AbstractSearchResultModel {

	private static final long serialVersionUID = 2628403363607798505L;

	private ArrayList<Entry> entries = new ArrayList<Entry>();

	EntrySearchResultModel(Model model) {
		super(model);
	}

	private String[] columns = new String[] { "Entry" };

	@Override
	public String getColumnName(int col) {

		return columns[col];

	}

	@Override
	public int getColumnCount() {
		return 1;
	}

	@Override
	public int getRowCount() {
		return entries.size();
	}

	@Override
	public Object getValueAt(int row, int col) {
		switch (col) {
		case 0:
			return entries.get(row);
			// case 1:
			// return features.get(row);
		}
		return null;

	}

	Entry getEntry(int row) {
		return entries.get(row);

	}

	void search(String text) {
		String lowerCaseText = text.toLowerCase();
		clear();
		for (Entry e : model.entries()) {
			if (e.getID().toLowerCase().contains(lowerCaseText))
				entries.add(e);

		}

		fireTableDataChanged();

	}

	void clear() {
		entries.clear();
		fireTableDataChanged();
	}
}